package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.JFedHighLevelException;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedException;
import be.iminds.ilabt.jfed.lowlevel.testbed_info.TestbedInfoSource;
import be.iminds.ilabt.jfed.lowlevel.api.ReputationService;
import be.iminds.ilabt.jfed.lowlevel.api.RestApi;
import be.iminds.ilabt.jfed.lowlevel.connection_pool.JFedConnectionProvider;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.highlevel.util.JavaFXLogger;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Date;
import java.util.List;

/**
 * User: twalcari
 * Date: 1/11/2016
 * Time: 11:38
 */
public class ListPendingReputationQuestionnairesTask extends ReputationTask {

    @Nonnull private final Slice slice;

    private List<ReputationService.SliceReputation> pendingQuestionnaires;

    public ListPendingReputationQuestionnairesTask(@Nonnull Slice slice,
                                                   @Nonnull JavaFXLogger logger,
                                                   @Nonnull JFedPreferences jFedPreferences,
                                                   @Nonnull TestbedInfoSource testbedInfoSource,
                                                   @Nonnull JFedConnectionProvider connectionProvider,
                                                   @Nonnull GeniUserProvider geniUserProvider) {
        super("Requesting pending reputation questionnaires",
                logger, jFedPreferences, testbedInfoSource,
                connectionProvider, geniUserProvider);

        this.slice = slice;
    }

    @Override
    protected void doReputationTask(TaskExecution taskExecution, ReputationService reputationService) throws JFedException {
        if (slice.getCreationDate() == null || slice.getExpirationDate() == null) {
            //noinspection ConstantConditions
            throw new RuntimeException("Cannot register Reputation questionaire: some slice info is missing. (this is an internal jFed error)"+
                    "urn="+slice.getUrn()+
                    (slice.getCreationDate() == null ? " cd" : "")+
                    (slice.getExpirationDate() == null ? " ed" : "")
            );
        }

        RestApi.RestReply<List<ReputationService.SliceReputation>> reply =
                reputationService.registerExperiment(
                        createConnection(),
                        geniUserProvider.getLoggedInGeniUser().getUserUrn(),
                        slice.getUrn(),
                        Date.from(slice.getCreationDate()),
                        Date.from(slice.getExpirationDate()));

        if (reply.getHttpCallDetails().getResultHttpStatusCode() != 200) {
            throw new JFedHighLevelException("Error while registering experiment to reputation service");
        }

        pendingQuestionnaires = reply.getValue();
    }

    @Nullable
    public List<ReputationService.SliceReputation> getPendingQuestionnaires() {
        return pendingQuestionnaires;
    }
}
